package com.foreverwin.mesnac.production.service.impl;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.mapper.SfcMapper;
import com.foreverwin.mesnac.meapi.model.Sfc;
import com.foreverwin.mesnac.production.mapper.SfcScrapMapper;
import com.foreverwin.mesnac.production.model.SfcScrap;
import com.foreverwin.mesnac.production.service.PodTemplateService;
import com.foreverwin.mesnac.production.service.SfcScrapService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.meext.MEServices;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.sap.me.production.ScrapDeleteServiceInterface;
import com.sap.me.production.ScrapSfcRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
/**
 * <p>
 * 产品报废 服务实现类
 * </p>
 *
 * @author zjw
 * @since 2021-07-30
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SfcScrapServiceImpl extends ServiceImpl<SfcScrapMapper, SfcScrap> implements SfcScrapService {


    @Autowired
    private SfcScrapMapper sfcScrapMapper;

    @Autowired
    private SfcMapper sfcMapper;

    @Autowired
    private PodTemplateService podTemplateService;

    @Override
    public IPage<SfcScrap> selectPage(FrontPage<SfcScrap> frontPage, SfcScrap sfcScrap) {
        QueryWrapper<SfcScrap> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(sfcScrap);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<SfcScrap> selectList(SfcScrap sfcScrap) {
        QueryWrapper<SfcScrap> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(sfcScrap);
        return super.list(queryWrapper);
    }


    @Override
    public List<SfcScrap> scrap(HashMap<String,Object> param) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        String reason = (String)param.get("reason");
        String site = (String)param.get("site");

        //类型
        List<String> _type = (List<String>)param.get("type");
        StringBuilder type = new StringBuilder();
        for (int i = 0; i < _type.size(); i++) {
            if(i == _type.size() - 1){
                type.append(_type.get(i));
            }else{
                type.append(_type.get(i)).append(",");
            }
        }


        String str = JSONUtil.toJsonStr(param.get("sfcScrap"));
        List<SfcScrap> _sfcScrapList = JSONArray.parseArray(str, SfcScrap.class);
        List<SfcScrap> sfcScrapsList = new ArrayList<>();


        ScrapDeleteServiceInterface scrapDeleteServiceInterface = null;
        //mes的公共接口
        try{
            scrapDeleteServiceInterface = MEServices.create("com.sap.me.production", "ScrapDeleteService", CommonMethods.getSite());
        }catch (Exception e){
            throw new BaseException("调用mes公共接口失败"+e.getMessage());
        }

        for (SfcScrap sfcScraps :
                _sfcScrapList) {
            SfcScrap sfcScrap = new SfcScrap();
            sfcScrap.setHandle(HandleEnum.SFC_SCRAP.getHandle(site,sfcScraps.getSfc()));
            sfcScrap.setSite(site);
            sfcScrap.setSfc(sfcScraps.getSfc());
            //设置数量用,并判断该sfc是否有效
            Sfc sfc = sfcMapper.selectById(HandleEnum.SFC.getHandle(site,sfcScrap.getSfc()));
            if(sfc == null){
                throw new BaseException("该sfc不存在，不能报废");
            }


            if(!StringUtil.isBlank(sfcScraps.getOperation())){
                String[] split = sfcScraps.getOperation().split("/");
                sfcScrap.setOperation(split[0]);
                sfcScrap.setStepId(split[1]);
            }
            sfcScrap.setType(type.toString());
            sfcScrap.setReason(reason);
            sfcScrap.setQty(sfc.getQty());
            sfcScrap.setCreatedUser(user);
            sfcScrap.setCreatedDateTime(now);
            sfcScrap.setModifiedUser(user);
            sfcScrap.setModifiedDateTime(now);



            //向生产派工那边发送数据
          //  podTemplateService.sendErp(sfcScrap.getSfc(),sfcScrap.getStepId(),BigDecimal.valueOf(0L),BigDecimal.valueOf(sfcScrap.getQty()));

            try{
                //调用mes公共接口，报废
                ScrapSfcRequest scrapSfcRequest = new ScrapSfcRequest();
                scrapSfcRequest.setSfcRef(sfc.getHandle());
                scrapSfcRequest.setQuantity(BigDecimal.valueOf(sfcScrap.getQty()));
                scrapDeleteServiceInterface.scrapSfc(scrapSfcRequest);
            }catch (Exception e){
                throw new BaseException("调用mes公共接口失败"+e.getMessage());
            }


            sfcScrapsList.add(sfcScrap);
        }
        //保存
        this.saveBatch(sfcScrapsList);
        return sfcScrapsList;
    }
}